package com.kdanmobile.admanager.admob

import android.app.Activity
import android.content.Context
import android.view.View
import com.google.android.gms.ads.MobileAds
import com.kdanmobile.admanager.*
import java.util.concurrent.CopyOnWriteArrayList

class AdmobAdManager : AdManager {
    private lateinit var context: Context
    private lateinit var interstitialAdManager: InterstitialAdManager
    private lateinit var nativeAdManager: NativeAdManager

    override var isInitialized = false
    override val onInitListeners = CopyOnWriteArrayList<Runnable>()

    override fun init(context: Context, adUnitId: String, volume: Float) {
        this.context = context

        MobileAds.initialize(context)
        MobileAds.setAppVolume(volume)

        interstitialAdManager = AdmobInterstitialAdManager()
        nativeAdManager = AdmobNativeAdManager(context)

        isInitialized = true
        onInitListeners.forEach { it.run() }
    }

    override fun requestInterstitial(activity: Activity, adUnitId: String) {
        interstitialAdManager.request(activity, adUnitId)
    }

    override fun showInterstitial(adUnitId: String): Boolean {
        return interstitialAdManager.show(adUnitId)
    }

    override fun destroyInterstitial(activity: Activity) {
        interstitialAdManager.destroy(activity)
    }

    override fun registerInterstitialAdListener(interstitialAdListener: InterstitialAdListener) {
        interstitialAdManager.registerListener(interstitialAdListener)
    }

    override fun unregisterInterstitialAdListener(interstitialAdListener: InterstitialAdListener) {
        interstitialAdManager.unregisterListener(interstitialAdListener)
    }

    override fun isInterstitialLoading(adUnitId: String): Boolean {
        return interstitialAdManager.isLoading(adUnitId)
    }

    override fun isInterstitialLoaded(adUnitId: String): Boolean {
        return interstitialAdManager.isLoaded(adUnitId)
    }

    override fun registerNativeAdListener(nativeAdNetworkListener: NativeAdNetworkListener) {
        nativeAdManager.networkListener += nativeAdNetworkListener
    }

    override fun unregisterNativeAdListener(nativeAdNetworkListener: NativeAdNetworkListener) {
        nativeAdManager.networkListener -= nativeAdNetworkListener
    }

    override fun requestNative(adUnitId: String, nativeAdViewBinder: NativeAdViewBinder) {
        nativeAdManager.request(adUnitId, nativeAdViewBinder)
    }

    override fun takeNativeAdView(adUnitId: String, nativeAdViewBinder: NativeAdViewBinder): View? {
        return nativeAdManager.takeAdView(adUnitId, nativeAdViewBinder)
    }
}