package com.kdanmobile.admanager.admob

import android.content.Context
import android.view.LayoutInflater
import android.view.View
import android.widget.Button
import android.widget.ImageView
import android.widget.TextView
import com.google.android.gms.ads.AdListener
import com.google.android.gms.ads.AdLoader
import com.google.android.gms.ads.AdRequest
import com.google.android.gms.ads.formats.*
import com.kdanmobile.admanager.NativeAdManager
import com.kdanmobile.admanager.NativeAdNetworkListener
import com.kdanmobile.admanager.NativeAdViewBinder
import java.util.concurrent.ConcurrentHashMap
import java.util.concurrent.CopyOnWriteArrayList

class AdmobNativeAdManager(private val context: Context) : NativeAdManager {

    override val networkListener = CopyOnWriteArrayList<NativeAdNetworkListener>()

    private val nativeAdMap = ConcurrentHashMap<String, UnifiedNativeAd>()

    override fun request(adUnitId: String, nativeAdViewBinder: NativeAdViewBinder) {
        AdLoader.Builder(context, adUnitId)
                .forUnifiedNativeAd {
                    nativeAdMap[adUnitId] = it
                    onLoaded(adUnitId)
                }.withAdListener(object : AdListener() {
                    override fun onAdFailedToLoad(p0: Int) {
                        super.onAdFailedToLoad(p0)
                        onFailedToLoad(adUnitId)
                    }
                }).withNativeAdOptions(NativeAdOptions.Builder().build())
                .build()
                .loadAd(AdRequest.Builder().build())
    }

    override fun takeAdView(adUnitId: String, nativeAdViewBinder: NativeAdViewBinder):View? {
        val ad = nativeAdMap.remove(adUnitId) ?: return null
        return buildAdView(ad, nativeAdViewBinder)
    }

    private fun onLoaded(adUnitId: String) {
        networkListener.forEach {
            it.onLoaded(adUnitId)
        }
    }

    private fun onFailedToLoad(adUnitId: String) {
        networkListener.forEach {
            it.onFailedToLoad(adUnitId)
        }
    }

    private fun buildAdView(ad: UnifiedNativeAd, nativeAdViewBinder: NativeAdViewBinder): View {
        val adView = LayoutInflater.from(context).inflate(nativeAdViewBinder.layout, null) as UnifiedNativeAdView
        adView.findViewById<MediaView>(nativeAdViewBinder.mainImageViewId)?.let {
            adView.mediaView = it
        }
        adView.findViewById<ImageView>(nativeAdViewBinder.iconImageViewId)?.let {
            ad.icon?.apply { it.setImageDrawable(this.drawable) }
            adView.iconView = it
        }
        adView.findViewById<TextView>(nativeAdViewBinder.titleTextViewId)?.let {
            ad.headline?.apply { it.text = this }
            adView.headlineView = it
        }
        adView.findViewById<TextView>(nativeAdViewBinder.textTextViewId)?.let {
            ad.body?.apply { it.text = this }
            adView.bodyView = it
        }
        adView.findViewById<Button>(nativeAdViewBinder.ctaButtonId)?.let {
            ad.callToAction?.apply { it.text = this }
            adView.callToActionView = it
        }
        adView.setNativeAd(ad)
        return adView
    }
}