package com.kdanmobile.admanager.mopub

import android.app.Activity
import com.kdanmobile.admanager.InterstitialAdListener
import com.kdanmobile.admanager.InterstitialAdManager
import com.mopub.mobileads.MoPubErrorCode
import com.mopub.mobileads.MoPubInterstitial
import java.util.concurrent.CopyOnWriteArrayList

class MoPubInterstitialAdManager: InterstitialAdManager {

    private val interstitialMap = HashMap<String, MoPubInterstitial>()
    private val activityInterstitialMap = HashMap<Activity, MoPubInterstitial>()
    private val adListeners = CopyOnWriteArrayList<InterstitialAdListener>()
    private val isLoadingMap = HashMap<String, Boolean>()

    override fun request(activity: Activity, adUnitId: String) {
        if (isLoaded(adUnitId)) return
        if (isLoading(adUnitId)) return

        var interstitial = interstitialMap[adUnitId]
        if (interstitial == null) {
            interstitial = MoPubInterstitial(activity, adUnitId).apply {
                interstitialAdListener = object : MoPubInterstitial.InterstitialAdListener {
                    override fun onInterstitialLoaded(interstitial: MoPubInterstitial?) {
                        adListeners.forEach { it.onLoaded(adUnitId) }
                        if (isLoadingMap[adUnitId] != null) {
                            isLoadingMap[adUnitId] = false
                        }
                    }

                    override fun onInterstitialShown(interstitial: MoPubInterstitial?) {
                        adListeners.forEach { it.onImpressed(adUnitId) }
                    }

                    override fun onInterstitialFailed(interstitial: MoPubInterstitial?, errorCode: MoPubErrorCode?) {
                        adListeners.forEach { it.onFailedToLoad(adUnitId) }
                        if (isLoadingMap[adUnitId] != null) {
                            isLoadingMap[adUnitId] = false
                        }
                    }

                    override fun onInterstitialDismissed(interstitial: MoPubInterstitial?) {
                        adListeners.forEach { it.onClosed(adUnitId) }
                    }

                    override fun onInterstitialClicked(interstitial: MoPubInterstitial?) {
                        adListeners.forEach { it.onClicked(adUnitId) }
                    }
                }
            }
            interstitialMap[adUnitId] = interstitial
            activityInterstitialMap[activity] = interstitial
        }
        interstitial?.load()
        isLoadingMap[adUnitId] = true
    }

    override fun show(adUnitId: String): Boolean {
        interstitialMap[adUnitId]?.let {
            if (it.isReady) {
                it.show()
                return true
            }
        }
        return false
    }

    override fun destroy(activity: Activity) {
        activityInterstitialMap.remove(activity)?.let { ad ->
            var id: String? = null
            interstitialMap.entries.forEach {
                if (it.value == ad) {
                    id = it.key
                    return@forEach
                }
            }
            interstitialMap.remove(id)
            isLoadingMap.remove(id)
            ad.destroy()
        }
    }

    override fun registerListener(interstitialAdListener: InterstitialAdListener) {
        adListeners.add(0, interstitialAdListener)
    }

    override fun unregisterListener(interstitialAdListener: InterstitialAdListener) {
        adListeners.remove(interstitialAdListener)
    }

    override fun isLoading(adUnitId: String): Boolean {
        isLoadingMap[adUnitId]?.let {
            return it
        }
        return false
    }

    override fun isLoaded(adUnitId: String): Boolean {
        interstitialMap[adUnitId]?.let {
            return it.isReady
        }
        return false
    }
}