package com.kdanmobile.admanager.mopub

import android.content.Context
import android.view.View
import com.kdanmobile.admanager.NativeAdManager
import com.kdanmobile.admanager.NativeAdNetworkListener
import com.kdanmobile.admanager.NativeAdViewBinder
import com.mopub.nativeads.*
import java.util.concurrent.ConcurrentHashMap
import java.util.concurrent.CopyOnWriteArrayList

class MoPubNativeAdManager(private val context: Context) : NativeAdManager {

    override val networkListener = CopyOnWriteArrayList<NativeAdNetworkListener>()

    private val nativeAdMap = ConcurrentHashMap<String, NativeAd>()

    override fun request(adUnitId: String, nativeAdViewBinder: NativeAdViewBinder) {
        val listener = object : MoPubNative.MoPubNativeNetworkListener {
            override fun onNativeLoad(nativeAd: NativeAd?) {
                if (nativeAd == null) return
                nativeAdMap[adUnitId] = nativeAd
                onLoaded(adUnitId)
            }

            override fun onNativeFail(errorCode: NativeErrorCode?) {
                onFailedToLoad(adUnitId)
            }
        }
        val viewBinder = buildViewBinder(nativeAdViewBinder)
        val native = MoPubNative(context, adUnitId, listener).apply {
            registerAdRenderer(MoPubStaticNativeAdRenderer(viewBinder))
        }
        native.makeRequest()
    }

    override fun takeAdView(adUnitId: String, nativeAdViewBinder: NativeAdViewBinder): View? {
        val nativeAd = nativeAdMap[adUnitId] ?: return null
        nativeAdMap.remove(adUnitId)
        val viewBinder = ViewBinder.Builder(0).build()
        val converterView = null
        val parent = null
        return AdapterHelper(context, 0, 3).getAdView(
                converterView,
                parent,
                nativeAd,
                viewBinder
        )
    }

    private fun onLoaded(adUnitId: String) {
        networkListener.forEach {
            it.onLoaded(adUnitId)
        }
    }

    private fun onFailedToLoad(adUnitId: String) {
        networkListener.forEach {
            it.onFailedToLoad(adUnitId)
        }
    }

    private fun buildViewBinder(nativeAdViewBinder: NativeAdViewBinder):ViewBinder {
        return ViewBinder.Builder(nativeAdViewBinder.layout).apply {
            iconImageId(nativeAdViewBinder.iconImageViewId)
            mainImageId(nativeAdViewBinder.mainImageViewId)
            titleId(nativeAdViewBinder.titleTextViewId)
            textId(nativeAdViewBinder.textTextViewId)
            callToActionId(nativeAdViewBinder.ctaButtonId)
            privacyInformationIconImageId(nativeAdViewBinder.privacyInformationIcon)
        }.build()
    }
}